import 'babel-polyfill';
import { randomBytes } from 'crypto';
import Redis from 'ioredis';

var asyncToGenerator = function (fn) {
  return function () {
    var gen = fn.apply(this, arguments);
    return new Promise(function (resolve, reject) {
      function step(key, arg) {
        try {
          var info = gen[key](arg);
          var value = info.value;
        } catch (error) {
          reject(error);
          return;
        }

        if (info.done) {
          resolve(value);
        } else {
          return Promise.resolve(value).then(function (value) {
            step("next", value);
          }, function (err) {
            step("throw", err);
          });
        }
      }

      return step("next");
    });
  };
};

var classCallCheck = function (instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
};

var createClass = function () {
  function defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  return function (Constructor, protoProps, staticProps) {
    if (protoProps) defineProperties(Constructor.prototype, protoProps);
    if (staticProps) defineProperties(Constructor, staticProps);
    return Constructor;
  };
}();







var get = function get(object, property, receiver) {
  if (object === null) object = Function.prototype;
  var desc = Object.getOwnPropertyDescriptor(object, property);

  if (desc === undefined) {
    var parent = Object.getPrototypeOf(object);

    if (parent === null) {
      return undefined;
    } else {
      return get(parent, property, receiver);
    }
  } else if ("value" in desc) {
    return desc.value;
  } else {
    var getter = desc.get;

    if (getter === undefined) {
      return undefined;
    }

    return getter.call(receiver);
  }
};

















var set = function set(object, property, value, receiver) {
  var desc = Object.getOwnPropertyDescriptor(object, property);

  if (desc === undefined) {
    var parent = Object.getPrototypeOf(object);

    if (parent !== null) {
      set(parent, property, value, receiver);
    }
  } else if ("value" in desc && desc.writable) {
    desc.value = value;
  } else {
    var setter = desc.set;

    if (setter !== undefined) {
      setter.call(receiver, value);
    }
  }

  return value;
};

var slicedToArray = function () {
  function sliceIterator(arr, i) {
    var _arr = [];
    var _n = true;
    var _d = false;
    var _e = undefined;

    try {
      for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
        _arr.push(_s.value);

        if (i && _arr.length === i) break;
      }
    } catch (err) {
      _d = true;
      _e = err;
    } finally {
      try {
        if (!_n && _i["return"]) _i["return"]();
      } finally {
        if (_d) throw _e;
      }
    }

    return _arr;
  }

  return function (arr, i) {
    if (Array.isArray(arr)) {
      return arr;
    } else if (Symbol.iterator in Object(arr)) {
      return sliceIterator(arr, i);
    } else {
      throw new TypeError("Invalid attempt to destructure non-iterable instance");
    }
  };
}();

var _class = function () {
  function _class(conn) {
    classCallCheck(this, _class);

    this.redis = new Redis(conn);
  }

  createClass(_class, [{
    key: 'save',
    value: function save(id, user, ttl) {
      if (ttl) {
        this.redis.set(id, JSON.stringify({ id: id, user: user }), 'ex', ttl);
      } else {
        this.redis.set(id, JSON.stringify({ id: id, user: user }));
      }
    }
  }, {
    key: 'get',
    value: function () {
      var _ref = asyncToGenerator(regeneratorRuntime.mark(function _callee(id) {
        var session;
        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                _context.next = 2;
                return this.redis.get(id);

              case 2:
                session = _context.sent;

                if (!session) {
                  _context.next = 5;
                  break;
                }

                return _context.abrupt('return', JSON.parse(session));

              case 5:
                return _context.abrupt('return', session);

              case 6:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      function get$$1(_x) {
        return _ref.apply(this, arguments);
      }

      return get$$1;
    }()
  }, {
    key: 'delete',
    value: function () {
      var _ref2 = asyncToGenerator(regeneratorRuntime.mark(function _callee2(id) {
        var session, user;
        return regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _context2.next = 2;
                return this.redis.get(id);

              case 2:
                session = _context2.sent;


                // Parse user object.
                user = JSON.parse(session.toString());

                // Delete session.

                _context2.next = 6;
                return this.redis.del(id);

              case 6:
                _context2.next = 8;
                return this.remove(user.id, id);

              case 8:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this);
      }));

      function _delete(_x2) {
        return _ref2.apply(this, arguments);
      }

      return _delete;
    }()
  }, {
    key: 'add',
    value: function () {
      var _ref3 = asyncToGenerator(regeneratorRuntime.mark(function _callee3(id, item) {
        return regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                this.redis.sadd(id, item);

              case 1:
              case 'end':
                return _context3.stop();
            }
          }
        }, _callee3, this);
      }));

      function add(_x3, _x4) {
        return _ref3.apply(this, arguments);
      }

      return add;
    }()
  }, {
    key: 'remove',
    value: function () {
      var _ref4 = asyncToGenerator(regeneratorRuntime.mark(function _callee4(id, item) {
        return regeneratorRuntime.wrap(function _callee4$(_context4) {
          while (1) {
            switch (_context4.prev = _context4.next) {
              case 0:
                this.redis.srem(id, item);

              case 1:
              case 'end':
                return _context4.stop();
            }
          }
        }, _callee4, this);
      }));

      function remove(_x5, _x6) {
        return _ref4.apply(this, arguments);
      }

      return remove;
    }()
  }, {
    key: 'all',
    value: function all(id) {
      var _this = this;

      return new Promise(function () {
        var _ref5 = asyncToGenerator(regeneratorRuntime.mark(function _callee5(resolve, reject) {
          var pipeline, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step;

          return regeneratorRuntime.wrap(function _callee5$(_context5) {
            while (1) {
              switch (_context5.prev = _context5.next) {
                case 0:
                  pipeline = _this.redis.pipeline();
                  _iteratorNormalCompletion = true;
                  _didIteratorError = false;
                  _iteratorError = undefined;
                  _context5.prev = 4;
                  _context5.next = 7;
                  return _this.redis.get(id);

                case 7:
                  _context5.t0 = Symbol.iterator;
                  _iterator = _context5.sent[_context5.t0]();

                case 9:
                  if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
                    _context5.next = 15;
                    break;
                  }

                  session = _step.value;

                  pipeline.get(session);

                case 12:
                  _iteratorNormalCompletion = true;
                  _context5.next = 9;
                  break;

                case 15:
                  _context5.next = 21;
                  break;

                case 17:
                  _context5.prev = 17;
                  _context5.t1 = _context5['catch'](4);
                  _didIteratorError = true;
                  _iteratorError = _context5.t1;

                case 21:
                  _context5.prev = 21;
                  _context5.prev = 22;

                  if (!_iteratorNormalCompletion && _iterator.return) {
                    _iterator.return();
                  }

                case 24:
                  _context5.prev = 24;

                  if (!_didIteratorError) {
                    _context5.next = 27;
                    break;
                  }

                  throw _iteratorError;

                case 27:
                  return _context5.finish(24);

                case 28:
                  return _context5.finish(21);

                case 29:
                  pipeline.exec(function (_, results) {
                    resolve(results.map(function (r) {
                      return r[1];
                    }));
                  });

                case 30:
                case 'end':
                  return _context5.stop();
              }
            }
          }, _callee5, _this, [[4, 17, 21, 29], [22,, 24, 28]]);
        }));

        return function (_x7, _x8) {
          return _ref5.apply(this, arguments);
        };
      }());
    }
  }, {
    key: 'clear',
    value: function clear(id) {
      var _this2 = this;

      return new Promise(function () {
        var _ref6 = asyncToGenerator(regeneratorRuntime.mark(function _callee6(resolve, reject) {
          var pipeline, _iteratorNormalCompletion2, _didIteratorError2, _iteratorError2, _iterator2, _step2;

          return regeneratorRuntime.wrap(function _callee6$(_context6) {
            while (1) {
              switch (_context6.prev = _context6.next) {
                case 0:
                  pipeline = _this2.redis.pipeline();
                  _iteratorNormalCompletion2 = true;
                  _didIteratorError2 = false;
                  _iteratorError2 = undefined;
                  _context6.prev = 4;
                  _context6.next = 7;
                  return _this2.redis.get(id);

                case 7:
                  _context6.t0 = Symbol.iterator;
                  _iterator2 = _context6.sent[_context6.t0]();

                case 9:
                  if (_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done) {
                    _context6.next = 15;
                    break;
                  }

                  session = _step2.value;

                  pipeline.del(session);

                case 12:
                  _iteratorNormalCompletion2 = true;
                  _context6.next = 9;
                  break;

                case 15:
                  _context6.next = 21;
                  break;

                case 17:
                  _context6.prev = 17;
                  _context6.t1 = _context6['catch'](4);
                  _didIteratorError2 = true;
                  _iteratorError2 = _context6.t1;

                case 21:
                  _context6.prev = 21;
                  _context6.prev = 22;

                  if (!_iteratorNormalCompletion2 && _iterator2.return) {
                    _iterator2.return();
                  }

                case 24:
                  _context6.prev = 24;

                  if (!_didIteratorError2) {
                    _context6.next = 27;
                    break;
                  }

                  throw _iteratorError2;

                case 27:
                  return _context6.finish(24);

                case 28:
                  return _context6.finish(21);

                case 29:
                  pipeline.exec(function (_, results) {
                    resolve(_this2.redis.del(key));
                  });

                case 30:
                case 'end':
                  return _context6.stop();
              }
            }
          }, _callee6, _this2, [[4, 17, 21, 29], [22,, 24, 28]]);
        }));

        return function (_x9, _x10) {
          return _ref6.apply(this, arguments);
        };
      }());
    }
  }]);
  return _class;
}();

var Bouncer = function Bouncer(store) {
  var _this = this;

  classCallCheck(this, Bouncer);

  this.store = store;

  this.session = function (req, res, next) {
    try {
      var authHeader = req.get('Authorization');
      if (authHeader) {
        var _authHeader$split = authHeader.split('Bearer '),
            _authHeader$split2 = slicedToArray(_authHeader$split, 2),
            _ = _authHeader$split2[0],
            token = _authHeader$split2[1];

        _this.store.get(token).then(function (session) {
          req.session = session;
          next();
        });
      } else {
        next();
      }
    } catch (err) {
      next(err);
    }
  };

  this.requireSession = function (req, res, next) {
    _this.session(req, res, function (err) {
      if (err) {
        next(err);
      } else if (req.session) {
        next();
      } else {
        res.status(401).end();
      }
    });
  };

  this.generate = function (user) {
    var ttl = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1209600;

    return new Promise(function (resolve, reject) {
      // Generate a unique token.
      randomBytes(64, function (err, buffer) {
        var token = buffer.toString('hex');

        // Save the session data to the store using a random token as the key.
        _this.store.save(token, user, ttl);

        resolve({ id: token, user: user });
      });
    });
  };

  this.destroy = function () {
    var _ref = asyncToGenerator(regeneratorRuntime.mark(function _callee(token) {
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _context.next = 2;
              return _this.store.delete(token);

            case 2:
              return _context.abrupt('return', _context.sent);

            case 3:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, _this);
    }));

    return function (_x2) {
      return _ref.apply(this, arguments);
    };
  }();
};

export { Bouncer, _class as RedisAdapter };
