import 'babel-polyfill'
import { randomBytes } from 'crypto'
import RedisAdapter from './adapters/redis'

class Bouncer {
  constructor (store) {
    this.store = store

    this.session = (req, res, next) => {
      try {
        const authHeader = req.get('Authorization')
        if (authHeader) {
          const [_, token] = authHeader.split('Bearer ')
          this.store.get(token).then(session => {
            req.session = session
            next()
          })
        } else {
          next()
        }
      } catch (err) {
        next(err)
      }
    }

    this.requireSession = (req, res, next) => {
      this.session(req, res, err => {
        if (err) {
          next(err)
        } else if (req.session) {
          next()
        } else {
          res.status(401).end()
        }
      })
    }

    this.generate = (user, ttl = 1209600) => {
      return new Promise((resolve, reject) => {
        // Generate a unique token.
        randomBytes(64, (err, buffer) => {
          const token = buffer.toString('hex')

          // Save the session data to the store using a random token as the key.
          this.store.save(token, user, ttl)

          resolve({ id: token, user })
        })
      })
    }

    this.destroy = async token => await this.store.delete(token)
  }
}

export { Bouncer, RedisAdapter }
