import test from 'ava'
import { Bouncer, RedisAdapter } from './dist/bouncer'
import httpMocks from 'node-mocks-http'

const redisAdapter = new RedisAdapter("redis://localhost:6379")
const bouncer = new Bouncer(redisAdapter)
const testUser = { id: 1, username: 'ian' }

// Clear Redis after each test.
test.afterEach(async t => await bouncer.store.redis.flushall())

test('bouncer.generate', async t => {
  const { id, user } = await bouncer.generate(testUser)
  t.truthy(id)
  t.is(user.id, testUser.id)
})

test('bouncer.destroy', async t => {
  const { id } = await bouncer.generate(testUser)
  await bouncer.destroy(id)
  t.falsy(await bouncer.store.get(id))
  t.falsy(await bouncer.store.get(testUser.id))
})

test('bouncer.session', async t => {
  const { id } = await bouncer.generate(testUser)
  const options = { headers: { Authorization: `Bearer ${id}`} }
  const request = httpMocks.createRequest(options)
  bouncer.session(request, null, () => {
    t.is(request.session.user.username, testUser.username)
  })
})

test('bouncer.requireSession', async t => {
  const request = httpMocks.createRequest()
  const response = httpMocks.createResponse()
  bouncer.requireSession(request, response, () => {
    t.is(response.statusCode, 401)
  })
})

test('bouncer.store.get', async t => {
  const { id } = await bouncer.generate(testUser)
  const { user } = await bouncer.store.get(id)
  t.is(user.id, testUser.id)
})
